@@ -20,12 +20,12 @@
 
 import errno
 import gc
+import io
 import logging
 import os
-import os
 import pprint
 import shutil
-import subprocess
+import sys
 import time
 
 from contextlib import closing
@@ -80,7 +80,7 @@ def skip_in_valgrind(f):
 class IOProcessTests(TestCase):
 
     def testMaxRequests(self):
-        proc = IOProcess(timeout=5, max_threads=1, max_queued_requests=1)
+        proc = IOProcess(timeout=10, max_threads=1, max_queued_requests=1)
         with closing(proc):
             t1 = Thread(target=proc.echo, args=("hello", 2))
             t2 = Thread(target=proc.echo, args=("hello", 2))
@@ -103,7 +103,7 @@ class IOProcessTests(TestCase):
                 t2.join()
 
     def testMaxRequestsAfterFillingThreadPool(self):
-        proc = IOProcess(timeout=5, max_threads=3, max_queued_requests=0)
+        proc = IOProcess(timeout=10, max_threads=3, max_queued_requests=0)
         with closing(proc):
             t1 = Thread(target=proc.echo, args=("hello", 2))
             t2 = Thread(target=proc.echo, args=("hello", 2))
@@ -127,12 +127,12 @@ class IOProcessTests(TestCase):
             t2.join()
 
     def testPing(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             self.assertEquals(proc.ping(), "pong")
 
     def test2SubsequentCalls(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             self.assertEquals(proc.ping(), "pong")
             self.assertEquals(proc.ping(), "pong")
@@ -141,13 +141,13 @@ class IOProcessTests(TestCase):
         data = """The Doctor: But I don't exist in your world!
                   Brigade Leader: Then you won't feel the bullets when we
                   shoot you."""  # (C) BBC - Doctor Who
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             self.assertEquals(proc.echo(data), data)
 
     def testUnicodeEcho(self):
         data = u'\u05e9\u05dc\u05d5\u05dd'
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             self.assertEquals(proc.echo(data), data)
 
@@ -159,7 +159,7 @@ class IOProcessTests(TestCase):
         threadnum = 10
         # We want to run all requests in parallel, so have one ioprocess thread
         # per client thread.
-        proc = IOProcess(timeout=2, max_threads=threadnum)
+        proc = IOProcess(timeout=10, max_threads=threadnum)
         with closing(proc):
             errors = []
             threads = []
@@ -184,7 +184,7 @@ class IOProcessTests(TestCase):
                                 [Stirs cup of tea with it]
                   Brigadier: I meant is there anything UNIT can do about this
                   space lightning business?"""  # (C) BBC - Doctor Who
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             self.assertEquals(proc.echo(data), data)
             self.assertTrue(proc.crash())
@@ -220,6 +220,7 @@ class IOProcessTests(TestCase):
                   The Doctor: Usually called "The Doctor." Or "The Caretaker."
                   Or "Get off this planet." Though, strictly speaking, that
                   probably isn't a name."""  # (C) BBC - Doctor Who
+        # Using smaller timeout to ensure the echo will time out.
         proc = IOProcess(timeout=1, max_threads=5)
         with closing(proc):
             try:
@@ -261,7 +262,7 @@ class IOProcessTests(TestCase):
                    Earthworm Jim: Thank you for cramming that delightful image
                                   into my brain, Peter.
                 '''  # (C) Universal Cartoon Studios - Earth Worm Jim
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             fd, path = mkstemp()
             try:
@@ -285,7 +286,7 @@ class IOProcessTests(TestCase):
                 os.unlink(path)
 
     def testStatFail(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             try:
                 proc.stat("/I do not exist")
@@ -296,7 +297,7 @@ class IOProcessTests(TestCase):
                 raise AssertionError("OSError was not raised")
 
     def testMissingArguemt(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             try:
                 proc._sendCommand("echo", {}, proc.timeout)
@@ -304,7 +305,7 @@ class IOProcessTests(TestCase):
                 self.assertEquals(e.errno, errno.EINVAL)
 
     def testNonExistingMethod(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             try:
                 proc._sendCommand("Implode", {}, proc.timeout)
@@ -312,7 +313,7 @@ class IOProcessTests(TestCase):
                 self.assertEquals(e.errno, errno.EINVAL)
 
     def testPathExists(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             fd, path = mkstemp()
             try:
@@ -322,12 +323,12 @@ class IOProcessTests(TestCase):
                 os.unlink(path)
 
     def testPathDoesNotExist(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             self.assertFalse(proc.pathExists("/I do not exist"))
 
     def testRename(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             fd, oldpath = mkstemp()
             newpath = oldpath + ".new"
@@ -345,7 +346,7 @@ class IOProcessTests(TestCase):
                     pass
 
     def testRenameFail(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             try:
                 proc.rename("/I/do/not/exist", "/Dsadsad")
@@ -356,7 +357,7 @@ class IOProcessTests(TestCase):
                 raise AssertionError("OSError was not raised")
 
     def testUnlink(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             fd, path = mkstemp()
             try:
@@ -369,7 +370,7 @@ class IOProcessTests(TestCase):
                     pass
 
     def testUnlinkFail(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             try:
                 proc.unlink("/I do not exist")
@@ -380,7 +381,7 @@ class IOProcessTests(TestCase):
                 raise AssertionError("OSError was not raised")
 
     def testLink(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             fd, oldpath = mkstemp()
             newpath = oldpath + ".new"
@@ -395,7 +396,7 @@ class IOProcessTests(TestCase):
                     pass
 
     def testLinkFail(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             try:
                 proc.link("/I/do/not/exist", "/Dsadsad")
@@ -406,7 +407,7 @@ class IOProcessTests(TestCase):
                 raise AssertionError("OSError was not raised")
 
     def testSymlink(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             fd, oldpath = mkstemp()
             newpath = oldpath + ".new"
@@ -423,7 +424,7 @@ class IOProcessTests(TestCase):
                     pass
 
     def testSymlinkFail(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             try:
                 proc.symlink("/Dsadsad", "/I/do/not/exist")
@@ -434,7 +435,7 @@ class IOProcessTests(TestCase):
                 raise AssertionError("OSError was not raised")
 
     def testChmod(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             fd, path = mkstemp()
             targetMode = os.W_OK | os.R_OK
@@ -448,7 +449,7 @@ class IOProcessTests(TestCase):
                 os.unlink(path)
 
     def testAccess(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             fd, path = mkstemp()
             try:
@@ -458,7 +459,7 @@ class IOProcessTests(TestCase):
                 os.unlink(path)
 
     def testChmodFail(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             try:
                 proc.chmod("/I/do/not/exist", 0)
@@ -468,56 +469,8 @@ class IOProcessTests(TestCase):
             else:
                 raise AssertionError("OSError was not raised")
 
-    def testReadfile(self, direct=False):
-        data = b'''The Doctor: Well... you could do that. Yeah, you could do
-                   that. Of course you could! But why? Look at these people,
-                   these human beings. Consider their potential! From the day
-                   they arrive on the planet, blinking, step into the sun,
-                   there is more to see than can ever be seen, more to do
-                   than-no, hold on. Sorry, that's The Lion King.
-                   But the point still stands: leave them alone!
-                   '''  # (C) BBC - Doctor Who
-        proc = IOProcess(timeout=1, max_threads=5)
-        with closing(proc):
-            fd, path = mkstemp(dir="/var/tmp")
-            try:
-                os.write(fd, data)
-                os.close(fd)
-                remoteData = proc.readfile(path, direct)
-                self.assertEquals(remoteData[:len(data)], data)
-            finally:
-                os.unlink(path)
-
-    def testReadfileWithDirectIO(self):
-        return self.testReadfile(True)
-
-    def testWritefile(self, direct=False):
-        data = b'''Jackie: I'm in my dressing gown.
-                   The Doctor: Yes, you are.
-                   Jackie: There's a strange man in my bedroom.
-                   The Doctor: Yes, there is.
-                   Jackie: Anything could happen.
-                   The Doctor: No. [walks away]'''  # (C) BBC - Doctor Who
-        # This test sometimes time out in the CI. On one failure, the write
-        # took 1.8 seconds inside ioprocess.
-        proc = IOProcess(timeout=5, max_threads=5)
-        with closing(proc):
-            fd, path = mkstemp(dir="/var/tmp")
-            try:
-                os.close(fd)
-                proc.writefile(path, data, direct)
-                with open(path, 'rb') as f:
-                    diskData = f.read()
-
-                self.assertEquals(diskData[:len(data)], data)
-            finally:
-                os.unlink(path)
-
-    def testWritefileWithDirectIO(self):
-        return self.testWritefile(True)
-
     def testListdir(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             path = mkdtemp()
             matches = []
@@ -538,7 +491,7 @@ class IOProcessTests(TestCase):
                 shutil.rmtree(path)
 
     def testGlob(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             path = mkdtemp()
             matches = []
@@ -557,7 +510,7 @@ class IOProcessTests(TestCase):
                 shutil.rmtree(path)
 
     def testRmdir(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             path = mkdtemp()
 
@@ -571,7 +524,7 @@ class IOProcessTests(TestCase):
                     pass
 
     def testMkdir(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             path = mkdtemp()
             shutil.rmtree(path)
@@ -586,7 +539,7 @@ class IOProcessTests(TestCase):
                     pass
 
     def testLexists(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             path = "/tmp/linktest.ioprocesstest"
             try:
@@ -600,19 +553,19 @@ class IOProcessTests(TestCase):
                 os.unlink(path)
 
     def testGlobNothing(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         with closing(proc):
             remoteMatches = proc.glob(os.path.join("/dsadasd", "*"))
             self.assertEquals(remoteMatches, [])
 
     def test_closed(self):
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         proc.close()
         self.assertRaises(Closed, proc.echo, "foo", 1)
 
     def test_close_terminates_process(self):
         for i in range(100):
-            proc = IOProcess(timeout=1, max_threads=5)
+            proc = IOProcess(timeout=10, max_threads=5)
             proc.close()
             self.assertFalse(os.path.exists("/proc/%d" % proc.pid),
                              "process %s did not terminate" % proc)
@@ -621,7 +574,7 @@ class IOProcessTests(TestCase):
         # Make inheritable file descriptor.
         with open(__file__) as my_file:
             clear_cloexec(my_file.fileno())
-            proc = IOProcess(timeout=1, max_threads=5)
+            proc = IOProcess(timeout=10, max_threads=5)
             with closing(proc):
                 # Wait until ready.
                 proc.ping()
@@ -632,7 +585,7 @@ class IOProcessTests(TestCase):
 
 
 def test_fsyncpath_file(tmpdir):
-    proc = IOProcess(timeout=1, max_threads=1)
+    proc = IOProcess(timeout=10, max_threads=1)
     with closing(proc):
         path = tmpdir.join("file")
         path.write("data")
@@ -642,7 +595,7 @@ def test_fsyncpath_file(tmpdir):
 
 
 def test_fsyncpath_directory(tmpdir):
-    proc = IOProcess(timeout=1, max_threads=1)
+    proc = IOProcess(timeout=10, max_threads=1)
     with closing(proc):
         # No easy way to test that we actually fsync this path. Lets just
         # call it to make sure it does not fail.
@@ -650,7 +603,7 @@ def test_fsyncpath_directory(tmpdir):
 
 
 def test_fsyncpath_missing(tmpdir):
-    proc = IOProcess(timeout=1, max_threads=1)
+    proc = IOProcess(timeout=10, max_threads=1)
     with closing(proc):
         with pytest.raises(OSError) as e:
             proc.fsyncPath("/no/such/file")
@@ -658,7 +611,7 @@ def test_fsyncpath_missing(tmpdir):
 
 
 def test_stat_file(tmpdir):
-    proc = IOProcess(timeout=1, max_threads=5)
+    proc = IOProcess(timeout=10, max_threads=5)
     with closing(proc):
         file = tmpdir.join("file")
         file.write(b"x" * 100)
@@ -667,14 +620,14 @@ def test_stat_file(tmpdir):
 
 
 def test_stat_dir(tmpdir):
-    proc = IOProcess(timeout=1, max_threads=5)
+    proc = IOProcess(timeout=10, max_threads=5)
     with closing(proc):
         dir = str(tmpdir)
         check_stat(proc.stat(dir), os.stat(dir))
 
 
 def test_stat_link(tmpdir):
-    proc = IOProcess(timeout=1, max_threads=5)
+    proc = IOProcess(timeout=10, max_threads=5)
     with closing(proc):
         src = tmpdir.join("src")
         src.write(b"x" * 100)
@@ -685,7 +638,7 @@ def test_stat_link(tmpdir):
 
 
 def test_stat_link_missing(tmpdir):
-    proc = IOProcess(timeout=1, max_threads=5)
+    proc = IOProcess(timeout=10, max_threads=5)
     with closing(proc):
         src = str(tmpdir.join("file"))
         link = str(tmpdir.join("link"))
@@ -698,7 +651,7 @@ def test_stat_link_missing(tmpdir):
 
 
 def test_lstat_file(tmpdir):
-    proc = IOProcess(timeout=1, max_threads=5)
+    proc = IOProcess(timeout=10, max_threads=5)
     with closing(proc):
         file = tmpdir.join("file")
         file.write(b"x" * 100)
@@ -707,14 +660,14 @@ def test_lstat_file(tmpdir):
 
 
 def test_lstat_dir(tmpdir):
-    proc = IOProcess(timeout=1, max_threads=5)
+    proc = IOProcess(timeout=10, max_threads=5)
     with closing(proc):
         dir = str(tmpdir)
         check_stat(proc.lstat(dir), os.lstat(dir))
 
 
 def test_lstat_link(tmpdir):
-    proc = IOProcess(timeout=1, max_threads=5)
+    proc = IOProcess(timeout=10, max_threads=5)
     with closing(proc):
         src = tmpdir.join("src")
         src.write(b"x" * 100)
@@ -725,7 +678,7 @@ def test_lstat_link(tmpdir):
 
 
 def test_lstat_link_missing(tmpdir):
-    proc = IOProcess(timeout=1, max_threads=5)
+    proc = IOProcess(timeout=10, max_threads=5)
     with closing(proc):
         src = str(tmpdir.join("file"))
         link = str(tmpdir.join("link"))
@@ -744,8 +697,73 @@ def check_stat(mystat, pystat):
         assert getattr(mystat, f) == getattr(pystat, f)
 
 
+@pytest.mark.parametrize("size", [0, 1, 512, 4096, 1024**2 + 1])
+def test_writefile(tmpdir, size):
+    data = b'x' * size
+    proc = IOProcess(timeout=10, max_threads=5)
+    with closing(proc):
+        path = str(tmpdir.join("file"))
+        proc.writefile(path, data)
+        with io.open(path, 'rb') as f:
+            written = f.read()
+        assert written == data
+
+
+# TODO: Assumes storage with 512 bytes sector size. Need to test with 4k
+# storage.
+@pytest.mark.parametrize("size", [0, 512, 1024**2])
+def test_writefile_direct_aligned(tmpdir, size):
+    data = b'x' * size
+    proc = IOProcess(timeout=10, max_threads=5)
+    with closing(proc):
+        path = str(tmpdir.join("file"))
+        proc.writefile(path, data, direct=True)
+        with io.open(path, 'rb') as f:
+            written = f.read()
+        assert written == data
+
+
+def test_writefile_direct_unaligned(tmpdir):
+    data = b'unaligned data'
+    proc = IOProcess(timeout=10, max_threads=5)
+    with closing(proc):
+        path = str(tmpdir.join("file"))
+        with pytest.raises(OSError) as e:
+            proc.writefile(path, data, direct=True)
+        assert e.value.errno == errno.EINVAL
+
+
+@pytest.mark.parametrize("size", [0, 1, 42, 512, 4096, 1024**2 + 1])
+def test_readfile(tmpdir, size):
+    data = b'x' * size
+    path = str(tmpdir.join("file"))
+    with io.open(path, "wb") as f:
+        f.write(data)
+
+    proc = IOProcess(timeout=10, max_threads=5)
+    with closing(proc):
+        read = proc.readfile(path)
+        assert read == data
+
+
+@pytest.mark.parametrize("size", [0, 1, 42, 512, 4096, 1024**2 + 1])
+def test_readfile_direct(tmpdir, size):
+    data = b'x' * size
+    path = str(tmpdir.join("file"))
+    with io.open(path, "wb") as f:
+        f.write(data)
+        os.fsync(f.fileno())
+
+    proc = IOProcess(timeout=10, max_threads=5)
+    with closing(proc):
+        read = proc.readfile(path, direct=True)
+        assert read == data
+
+
 class TestWeakerf(TestCase):
 
+    @pytest.mark.xfail(
+        sys.version_info[:2] == (3, 7), reason="fails in python 3.7")
     def test_close_when_unrefed(self):
         """Make sure there is nothing keepin IOProcess strongly referenced.
 
@@ -753,7 +771,7 @@ class TestWeakerf(TestCase):
         work we need to make sure it doesn't prevent IOProcess from being
         garbage collected.
         """
-        proc = IOProcess(timeout=1, max_threads=5)
+        proc = IOProcess(timeout=10, max_threads=5)
         proc = ref(proc)
 
         end = elapsed_time() + 5.0
@@ -795,7 +813,7 @@ class LoggingTests(TestCase):
 
     def test_partial_logs(self):
         threads = []
-        proc = IOProcess(timeout=1, max_threads=10)
+        proc = IOProcess(timeout=10, max_threads=10)
         proc._sublog = FakeLogger()
 
         def worker():
